;The G.E.C.K. PowerUp for New Vegas NVSE plugin
;File version 0.10.0 (from 0.9 in 1.3 beta)
;Written by Fedor Uvarov
;Patched to support G.E.C.K 1.4 by H. LE PORS ( 17 November 2011)
;       Restore all warnings for test
;       Add activate ESM

;This source code is compilable with Flat Assembler (http://flatassmebler.net)
;I use version 1.69.25  --- Patched using v1.69.31 ---
;                       ------------------------------
;Please note that there is a certain bug in FASM include files, on which this source may be dependent
;(namely, MENUITEMINFO definition). This will be fixed in PowerUp 0.2.0 by redifining this structure.

format PE GUI 4.0 DLL
entry DLLEntry

include 'win32a.inc'
include 'macro\if.inc'

;Parts
include 'library\warnings.asm'
include 'library\scripts-warnings.asm'
include 'library\formlisteditor-dblclk.asm'
include 'library\spellcheck.asm'
include 'library\windows.asm'
include 'library\scripts-compilation-bypass.asm'
include 'library\formlisteditor-edit.asm'
include 'library\printf.asm'
include 'library\report-procs.asm'
include 'library\messagebox.asm'
include 'library\markers-hack.asm'
include 'library\activateesm.asm'

;CONSTANTS
LANG = LANG_ENGLISH + SUBLANG_DEFAULT
EXITCODE_ABORT = 255
REPORTED_LIB_VERSION = 6 ;1 for 0.1.0, 2 for 0.1.1, 3 for 0.1.2, 4 for 0.1.3, 4 for 0.1.3.1, 5 for 0.1.4

;COMMON MACROS AND STRUCTURES
;Duplicates <numdwords> DWORDs on the stack, but copies them over current
;DWORD. I.e. <data>(ESP points here)<4 bytes> -> <data><4 bytes><data (ESP points to last 4 bytes of data)>
;Uses EAX, ECX
macro CopyStackDWORDs numdwords {
	mov ecx, numdwords
	local copyloop
	copyloop: lea eax, [esp+numdwords*4]
	push DWORD [eax]
	loop copyloop
}

;Copied from WINDOWS.INC for 32 bit MASM (Version 1.30 Release 7 March 2006)
struct CRITICAL_SECTION
    DebugInfo	    dd ?
    LockCount	    dd ?
    RecursionCount  dd ?
    OwningThread    dd ?
    LockSemaphore   dd ?
    SpinCount	    dd ?
ends


section '.data' data readable writeable
OldProtection dd ?
ErrorCaption db 'The GECK 1.4 PowerUp Error', 0
ErrorMsg db 'Patching failed! Exiting now.', 0
GECKPowerUp db "The GECK 1.4 PowerUp", 0
NVSELibName db "nvse_editor_1_4.dll", 0
ErrorNoNVSE db "Could not load NVSE library (nvse_editor_1_4.dll)", 0
OldProtectionOffset dd ?
OldProtectionLength dd ?
hMainMenu dd 0
bReportCriticalSectionInitialized db 0
ReportCriticalSection CRITICAL_SECTION ;Critical section object. Reports use this CS because they can end up patching the code.
db sizeof.CRITICAL_SECTION

WARNINGS_DATA
SCRIPTS_WARNINGS_DATA
WINDOWS_DATA
FORMLISTEDITOR_DBLCLK_DATA
SPELLCHECK_DATA
SCRIPTS_COMPILATION_BYPASS_DATA
PRINTF_DATA
REPORTPROCS_DATA
MARKERHACK_DATA
CRASH_DATA


section '.code' code readable executable
proc DLLEntry hDLL, dwReason, dwReserved
	.if [dwReason] = DLL_PROCESS_DETACH
		.if [hMainMenu]
			invoke DestroyMenu, [hMainMenu]
		.endif
		.if [bReportCriticalSectionInitialized]
			invoke DeleteCriticalSection, ReportCriticalSection
		.endif
	.endif
	mov eax, TRUE
	ret
endp

proc GetLibVersionAlpha
	mov eax, REPORTED_LIB_VERSION
	ret
endp

proc PatchAbort
	invoke MessageBoxEx, 0, ErrorMsg, ErrorCaption, MB_OK + MB_ICONERROR, LANG
	invoke ExitProcess, EXITCODE_ABORT
	ret
endp

;Sets memory protection to PAGE_EXECUTE_READWRITE, saves old protection
;Only use MakeMemWriteable, then RestoreMemProtection after all changes,
;otherwise old protection data will be lost
proc MakeMemWriteable, StartOffset, TargetLength
	push eax
	mov eax, [StartOffset]
	mov [OldProtectionOffset], eax
	mov eax, [TargetLength]
	mov [OldProtectionLength], eax
	invoke VirtualProtect, [StartOffset], [TargetLength], PAGE_EXECUTE_READWRITE, OldProtection
	test eax, eax
	jnz .exitproc
	call PatchAbort
	.exitproc:
	pop eax
	ret
endp

;Restores memory protection
;Only use once after each MakeMemWriteable call
proc RestoreMemProtection
	push eax
	invoke VirtualProtect, [OldProtectionOffset], [OldProtectionLength], [OldProtection], OldProtection
	test eax, eax
	jnz .exitproc
	call PatchAbort
	.exitproc:
	pop eax
	ret
endp

;Moves TargetLength NOPs to TargetOffset. Synchronized.
proc PatchWithNops TargetOffset, TargetLength
	stdcall MakeMemWriteable, [TargetOffset], [TargetLength]
	push edi
	mov edi, [TargetOffset]
	mov ecx, [TargetLength]
	mov al, 090h
	.dopatch: stosb
	loop .dopatch
	pop edi
	ret
endp

;Appends TailString to HeadString and copies them both to Buffer. Fails if BufferSize is enough to hold both.
;Strings must be 0-terminated. Result will also be 0-terminated, unless the function fails.
;Returns 0 if unsuccessful, size of the result otherwise
proc ConcatStrings HeadString, TailString, Buffer, BufferSize
	push esi
	push edi
	mov edi, [Buffer]
	
	mov ecx, [BufferSize]
	test ecx, ecx
	jz .exit_fail	;No buffer, no game
	
	;Copying HeadString to buffer
	mov esi, [HeadString]
	.copyhead:
	lodsb
	test al, al
	jz .end_copyhead
	stosb
	loop .copyhead
	jmp .exit_fail	;If we haven't finished yet, nothing can save us
	
	.end_copyhead:
	;No check: we still have at least 1 byte in the buffer
	mov esi, [TailString]
	.copytail:
	lodsb
	stosb	;We need 0 at the end of the result
	test al, al
	jz .end_copytail
	loop .copytail
	jmp .exit_fail
	
	.end_copytail:	;Success
	dec ecx ;Ending 0 not counted
	mov eax, [BufferSize]
	sub eax, ecx
	jmp .exitproc
	
	.exit_fail:
	xor eax, eax
	
	.exitproc:
	pop edi
	pop esi
	ret
endp

;Allocates buffer on the stack (with 4-byte alignment) and calls ConcatStrings
;EAX is 0 on error (stack is cleaned then) or size of the buffer on success
;Uses EAX, ECX, EDX
;On success ESP is at the start of the buffer, otherwise where it was
macro ConcatStringsInStack HeadString, HeadStringSize, TailString, TailStringSize {
	;We'll declare local labels here
	local concat_succeeded
	local concat_end
	
	;Calculating size+alignment
	mov eax, HeadStringSize
	add eax, TailStringSize
	dec eax ;Strings contain zeros at their ends, one of them will be skipped
	xor edx, edx
	mov ecx, 4
	div ecx
	mov eax, edx
	add eax, HeadStringSize
	add eax, TailStringSize
	dec eax
	sub esp, eax
	
	;Calling ConcatStrings
	push eax
	mov ecx, eax
	lea eax, [esp+4]
	stdcall ConcatStrings, HeadString, TailString, eax, ecx
	test eax, eax
	jnz concat_succeeded
	
	;Failed
	pop eax
	add esp, eax
	xor eax, eax
	jmp concat_end
	
	;Succeeded
	concat_succeeded:
	pop eax
	
	;End of macro
	concat_end:
}

;Extra procs
PRINTF_PROCS

proc ApplyPatch bNeedNVSE
	cmp [bNeedNVSE], 1
	jne ApplyPatch.continue_load
	
	invoke LoadLibrary, NVSELibName
	test eax, eax
	jnz ApplyPatch.continue_load
	
	invoke MessageBoxEx, 0, ErrorNoNVSE, ErrorCaption, MB_OK + MB_ICONWARNING, LANG
	call PatchAbort
	
	ApplyPatch.continue_load:
	invoke InitializeCriticalSection, ReportCriticalSection
	
	SCRIPTS_WARNINGS_PATCHES	;Warnings restoration
	WINDOWS_PATCHES ;Windows management patches
	FORMLISTEDITOR_DBLCLK_PATCHES ;Applying the form list double click patch
	SPELLCHECK_PATCHES ;Spellchecker patch
	SCRIPTS_COMPILATION_BYPASS_PATCHES ;Script compilation bypass
	REPORTPROCS_PATCHES	;Report functions
	MESSAGEBOX_PATCHES	;MessageBox language fix
	MARKERHACK_PATCHES	;Makes RoomMarkers and PortalMarkers visible
	CRASH_PATCHES		;Makes ESM activable (and no crash on load/save)

	;Exiting
	xor eax, eax
	ret
endp

; NVSE plugin API
struct NVSEInterface
       nvseVersion	  dd ?
       runtimeVersion	  dd ?
       editorVersion	  dd ?
       isEditor 	  dd ?
       RegisterCommand	  dd ?
       SetOpcodeBase	  dd ?
       QueryInterface	  dd ?
ends

struct PluginInfo
       infoVersion	dd ?
       name		dd ?
       version		dd ?
ends

proc NVSEPlugin_Query c nvse, info
     ; fill out info struct
     mov eax, [info]
     mov [eax+PluginInfo.infoVersion], 1
     mov [eax+PluginInfo.name], GECKPowerUp
     mov [eax+PluginInfo.version], REPORTED_LIB_VERSION

     ; check context
     mov eax, [nvse]
     cmp [eax+NVSEInterface.isEditor], 0
     jz .badVersion

     ;     cmp [eax+NVSEInterface.editorVersion], 0x01001060 ; 1.1.0.262
     ;     cmp [eax+NVSEInterface.editorVersion], 0x03001C40 ; 1.3.0.452
     cmp [eax+NVSEInterface.editorVersion], 0x04002060	; 1.4.0.518
     jnz .badVersion

     ; version checks pass
     mov eax, 1
     ret

     .badVersion:
     xor eax, eax
     ret
endp

proc NVSEPlugin_Load c nvse
     stdcall ApplyPatch, 0
     mov eax, 1
     ret
endp

;PATCH CALLBACKS
SCRIPTS_WARNINGS_CALLBACKS
WINDOWS_CALLBACKS
FORMLISTEDITOR_DBLCLK_CALLBACKS
SPELLCHECK_CALLBACKS
SCRIPTS_COMPILATION_BYPASS_CALLBACKS
REPORTPROCS_CALLBACKS
MESSAGEBOX_CALLBACKS
MARKERHACK_CALLBACKS
CRASH_CALLBACKS

section '.idata' import data readable
library kernel, 'kernel32.dll',\
	user, 'user32.dll',\
	comdlg, 'comdlg32.dll'

import user,\
	MessageBoxEx, 'MessageBoxExA',\
	GetDlgItem, 'GetDlgItem',\
	RegisterClass, 'RegisterClassA',\
	CreateWindowEx, 'CreateWindowExA',\
	DestroyMenu, 'DestroyMenu',\
	LoadMenu, 'LoadMenuA',\
	InsertMenuItem, 'InsertMenuItemA',\
	GetMenu, 'GetMenu',\
	SetMenu, 'SetMenu',\
	SetMenuItemInfo, 'SetMenuItemInfoA',\
	CreateDialogParam, 'CreateDialogParamA',\
	PostMessage, 'PostMessageA',\
	SendMessage, 'SendMessageA',\
	DrawMenuBar, 'DrawMenuBar'

import kernel,\
	VirtualProtect, 'VirtualProtect',\
	ExitProcess, 'ExitProcess',\
	lstrcmp, 'lstrcmpA',\
	GetModuleHandle, 'GetModuleHandleA',\
	GetCurrentProcess, 'GetCurrentProcess',\
	VirtualAllocEx, 'VirtualAllocEx',\
	VirtualFreeEx, 'VirtualFreeEx',\
	LoadLibrary, 'LoadLibraryA',\
	InitializeCriticalSection, 'InitializeCriticalSection',\
	EnterCriticalSection, 'EnterCriticalSection',\
	LeaveCriticalSection, 'LeaveCriticalSection',\
	DeleteCriticalSection, 'DeleteCriticalSection'
	
import comdlg,\
	GetSaveFileName, 'GetSaveFileNameA'

	
section '.edata' export data readable
export 'nvse_plugin_geckpu-nv-14.dll',\
	GetLibVersionAlpha, 'GetLibVersionAlpha',\
	ApplyPatch, 'ApplyPatch',\
	NVSEPlugin_Query, 'NVSEPlugin_Query',\
	NVSEPlugin_Load, 'NVSEPlugin_Load'

section '.reloc' fixups data readable


section '.rsrc' resource data readable
	directory RT_VERSION, versions
	resource versions, 1, LANG_ENGLISH + SUBLANG_DEFAULT, vinfo
	versioninfo vinfo, VOS__WINDOWS32, VFT_DLL, VFT2_UNKNOWN,\
		LANG_ENGLISH + SUBLANG_DEFAULT, 0,\
		'FileDescription', 'The G.E.C.K. 1.4 PowerUp NV Main Library',\
		'FileVersion', '0.1.7.2',\
		'ProductName', 'The G.E.C.K. 1.4 PowerUp for Fallout: New Vegas',\
		'ProductVersion', '0.1.7.2'